package com.ut.mini;

import java.util.HashMap;
import java.util.Map;

import android.app.Application;
import android.content.Context;
import android.os.Build;

import com.ut.mini.base.UTMCStatConfig;
import com.ut.mini.base.UTMIVariables;
import com.ut.mini.core.UTMCVariables;
import com.ut.mini.core.appstatus.UTMCAppStatusRegHelper;
import com.ut.mini.core.sign.IUTRequestAuthentication;
import com.ut.mini.crashhandler.IUTCrashCaughtListner;
import com.ut.mini.crashhandler.UTMiniCrashHandler;
import com.ut.mini.internal.UTOriginalCustomHitBuilder;
import com.ut.mini.internal.UTTeamWork;
import com.ut.mini.log.UTMCLogger;
import com.ut.mini.plugin.UTPluginMgr;
import com.ut.mini.sdkevents.UTMI1010_2001Event;
import com.ut.mini.utils.UTMCStringUtils;
/**
 * UT统计入口类
 * @author daichenjie
 * <pre>
 * 发送打点，首先需要获取一个UTTracker实例。
 * 
 * 获取UTTracker打点实例的方式有2种途径：
 *    1)通过UTAnalytics.getInstance().getDefaultTracker.
 *    2)通过UTAnalytics.getInstance().getTracker(String aTrackId)
 *    
 * 这里TrackId是一个什么概念呢？
 *    1)独立的Tracker共享defaultTracker的基础数据，如appkey，channel，用户帐号，设备信息等，你
 *    	可以认为独立的Tracker就多了一个TrackId，其它的一致。
 *    2)所有Tracker打出来的数据，我们会在reserves字段中，增加_tkid=TrackId值这个内容，用来和
 *    	defaultTracker的数据做一个区别
 *    
 * 那么，怎么完成一个打点呢？(下面以一个播放音乐自定义事件为例)
 * 
 *    1)通过defaultTracker打点
 *         //构造打点的map
 *         UTCustomHitBuilder lHitBuilder = new UTHitBuilders.UTCustomHitBuilder(
 *				"play_music");
 *         lHitBuilder.setProperty("type", "rock");
 *         lHitBuilder.setProperty("title", "wonderful tonight");
 *         //通过defaultTracker发送打点
 *         UTAnalytics.getInstance().getDefaultTracker().send(lHitBuilder.build());
 *
 *    2)通过独立的Tracker打点
 *         //构造打点的map
 *         UTCustomHitBuilder lHitBuilder = new UTHitBuilders.UTCustomHitBuilder(
 *				"play_music");
 *         lHitBuilder.setProperty("type", "rock");
 *         lHitBuilder.setProperty("title", "wonderful tonight");
 *         //通过独立Tracker发送打点
 *         UTTracker lTracker = UTAnalytics.getInstance().getTracker("your track id");
 *         lTracker.send(lHitBuilder.build());
 *</pre>
 */
public class UTAnalytics {

	private static UTAnalytics s_instance = null;
	private UTTracker mDefaultTracker;
	private Map<String, UTTracker> mTrackerMap = new HashMap<String, UTTracker>();

	private UTAnalytics() {
		// 注入消息通知到MINICore
		if (Build.VERSION.SDK_INT < Build.VERSION_CODES.ICE_CREAM_SANDWICH) {
			// 注册插件，完成1010，2001的一些事项
			UTMI1010_2001Event l1010and2001EventInstance = new UTMI1010_2001Event();
			UTPluginMgr.getInstance().registerPlugin(l1010and2001EventInstance, false);
			UTMIVariables.getInstance().setUTMI1010_2001EventInstance(l1010and2001EventInstance);
		} else {
			UTMI1010_2001Event l1010and2001EventInstance = new UTMI1010_2001Event();
			UTMCAppStatusRegHelper
					.registerAppStatusCallbacks(l1010and2001EventInstance);
			UTMIVariables.getInstance().setUTMI1010_2001EventInstance(l1010and2001EventInstance);
		}
	}

	/**
	 * 设置SDK执行需要的Context
	 * @param aContext
	 */
	public void setContext(Context aContext) {
		UTMCStatConfig.getInstance().setContext(aContext);

		if (null != aContext) {
			// 如果用户直接通过UTCrashHandler关闭这个CrashHandler，那么我们不再次初始化它
			if (!UTMiniCrashHandler.getInstance().isTurnOff()) {
				UTMiniCrashHandler.getInstance().turnOn(aContext);
			}
			// 因为context是一个必要条件，所以，这里可以启动TeamWork的合作的模块
			UTTeamWork.getInstance().initialized();
		}
	}

	/**
	 * 设置Application实例，完成4.0以上自动页面以及应用使用时长运算
	 * @param aApplicationInstance 若没有Application实例，那么需要用ApplicationContext代替
	 * 
	 */
	public void setAppApplicationInstance(Application aApplicationInstance) {
		UTMCStatConfig.getInstance().setAppApplicationInstance(
				aApplicationInstance);
	}

	/**
	 * 获取UTAnalytics实例
	 * @return 需要先调用setRequestAuthentication设置签名验证方式，返回的值才不为null
	 */
	public synchronized static UTAnalytics getInstance() {
		if (null == s_instance) {
			s_instance = new UTAnalytics();
		}
		return s_instance;
	}

	/**
	 * 获取默认埋点实例
	 * @return
	 */
	public synchronized UTTracker getDefaultTracker() {
		if (null == mDefaultTracker) {
			if (null != UTMCStatConfig.getInstance()
					.getRequestAuthenticationInstance()) {
				UTTracker lTracker = new UTTracker();
				mDefaultTracker = lTracker;
			}
		}
		if(null == mDefaultTracker){
			UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG, "getDefaultTracker error", "Fatal Error,must call setRequestAuthentication method first.");
		}
		return mDefaultTracker;
	}

	/**
	 * 设置SDK发送请求验证类型
	 * @param aRequestAuthenticationInstance 不能为null，设置失败会导致服务端拒绝数据
	 */
	public void setRequestAuthentication(
			IUTRequestAuthentication aRequestAuthenticationInstance) {
		if(null == aRequestAuthenticationInstance){
			UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG, "setRequestAuthentication", "Fatal Error,pRequestAuth must not be null.");
		}
		UTMCStatConfig.getInstance().setRequestAuthenticationInstance(
				aRequestAuthenticationInstance);
	}

	/**
	 * 设置应用版本，如果xml中的versionname不能满足需求(关联报表指标：app版本维度)
	 * @param aAppVersion
	 */
	public void setAppVersion(String aAppVersion) {
		UTMCStatConfig.getInstance().setAppVersion(aAppVersion);
	}

	/**
	 * 获取更多的埋点实例（可以理解为插件的模式）
	 * @param aTrackId	实例ID（例如：插件ID）
	 * @return
	 */
	public synchronized UTTracker getTracker(String aTrackId) {
		if (!UTMCStringUtils.isEmpty(aTrackId)) {
			if (mTrackerMap.containsKey(aTrackId)) {
				return mTrackerMap.get(aTrackId);
			} else {
				UTTracker lTracker = new UTTracker();
				lTracker.setTrackId(aTrackId);
				mTrackerMap.put(aTrackId, lTracker);
				return lTracker;
			}
		}else{
			UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG, "getTracker", "TrackId is null.");
		}
		return null;
	}

	/**
	 * 设置渠道，如91(关联报表指标：渠道统计)
	 * @param aChannel
	 */
	public void setChannel(String aChannel) {
		UTMCStatConfig.getInstance().setChannel(aChannel);
	}

	/**
	 * 关闭crash采集
	 */
	public void turnOffCrashHandler() {
		UTMiniCrashHandler.getInstance().turnOff();
	}

	/**
	 * 设置crash触发之后，回调通知
	 * @param aListener
	 */
	public void setCrashCaughtListener(IUTCrashCaughtListner aListener) {
		UTMiniCrashHandler.getInstance().setCrashCaughtListener(aListener);
	}

	/**
	 * 打开调试log显示
	 */
	public void turnOnDebug() {
		UTMCStatConfig.getInstance().turnOnDebug();
	}

	/**
	 * 更新用户账号信息(关联报表指标：活跃用户数)
	 * @param aUsernick	昵称
	 * @param aUserid	数字ID
	 */
	public void updateUserAccount(String aUsernick, String aUserid) {
		UTMCStatConfig.getInstance().updateUserAccount(aUsernick, aUserid);
	}

	/**
	 * 发送用户注册事件(关联报表指标：注册用户数)
	 * 
	 * @param aUsernick
	 *            用户昵称
	 */
	public void userRegister(String aUsernick) {
		if (!UTMCStringUtils.isEmpty(aUsernick)) {
			UTTracker lTracker = this.getDefaultTracker();
			if(null != lTracker){
				UTOriginalCustomHitBuilder lHitBuilder = new UTOriginalCustomHitBuilder(
						"UT", 1006, aUsernick, null, null, null);
				lTracker.send(lHitBuilder.build());
			}else{
				UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG, "Record userRegister event error", "Fatal Error,must call setRequestAuthentication method first.");
			}
		}else{
			UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG, "userRegister", "Fatal Error,usernick can not be null or empty!");
		}
	}

	/**
	 * 更新session信息,传入的信息会在每条日志的reserves字段携带
	 * session信息清除逻辑:应用切后台超过30秒再进入前台会清除之前的session信息
	 * @param aMap
	 *            session信息
	 */
	public void updateSessionProperties(Map<String,String> aMap){
		Map<String,String> lOldMap = UTMCVariables.getInstance().getSessionProperties();
		Map<String,String> lNewMap = new HashMap<String,String>();
		if(null != lOldMap){
			lNewMap.putAll(lOldMap);
		}
		lNewMap.putAll(aMap);
		UTMCVariables.getInstance().setSessionProperties(lNewMap);
	}
	
	/**
	 * 关闭页面自动采集功能
	 */
	public void turnOffAutoPageTrack(){
		UTPageHitHelper.getInstance().turnOffAutoPageTrack();
	}
	
}
