package com.ut.mini;

import java.util.HashMap;
import java.util.Map;

import com.ut.mini.base.UTLogFieldsScheme;
import com.ut.mini.log.UTMCLogger;
import com.ut.mini.utils.UTMCMapUtils;
import com.ut.mini.utils.UTMCStringUtils;
/**
 * UT打点辅助类的容器类
 * @author daichenjie
 * <pre>
 * 内部的子类的使命都在构造最完美的打点数据结构体。
 * 
 * UTPageHitBuilder：页面打点
 * UTCustomeHitBuilder：自定义事件打点
 * </pre>
 */
public class UTHitBuilders {

	/**
	 * 打点基类，不建议直接使用
	 * @author daichenjie
	 *
	 */
	public static class UTHitBuilder {

		public final static String FIELD_PAGE = "_field_page";
		public final static String FIELD_EVENT_ID = "_field_event_id";
		public final static String FIELD_ARG1 = "_field_arg1";
		public final static String FIELD_ARG2 = "_field_arg2";
		public final static String FIELD_ARG3 = "_field_arg3";
		public final static String FIELD_ARGS = "_field_args";

		private Map<String, String> mHitMap = new HashMap<String, String>();

		public UTHitBuilder() {
			if(!mHitMap.containsKey(FIELD_PAGE)){
				mHitMap.put(FIELD_PAGE, "UT");
			}
		}

		/**
		 * 给打点添加属性对
		 * @param aKey		key，不能为空和null
		 * @param aValue	value，不能为null
		 * @return
		 */
		public UTHitBuilder setProperty(String aKey, String aValue) {
			if (!UTMCStringUtils.isEmpty(aKey) && null != aValue) {
				if(this.mHitMap.containsKey(aKey)){
					this.mHitMap.remove(aKey);
				}
				this.mHitMap.put(aKey, aValue);
			}else{
				UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG, "setProperty", "key is null or key is empty or value is null,please check it!");
			}
			return this;
		}

		/**
		 * 给打点设置多对属性对
		 * @param aProperties	多个属性对，如果key为null或空字符，会被自动丢弃掉。
		 * @return
		 */
		public UTHitBuilder setProperties(Map<String, String> aProperties) {
			if (null != aProperties) {
				this.mHitMap.putAll(aProperties);
			}
			return this;
		}

		/**
		 * 获取指定的属性值
		 * @param aKey	属性key
		 * @return
		 */
		public String getProperty(String aKey) {
			if (null != aKey) {
				if (this.mHitMap.containsKey(aKey)) {
					return this.mHitMap.get(aKey);
				}
			}
			return null;
		}

		/**
		 * 构造打点需要的map，通过UTTracker发送
		 * @return
		 */
		public Map<String, String> build() {
			// Fix：用临时的map来产生打点的map，这样可以多次build不会有影响。
			// 再子类重写了build方法的话，需要多注意这个问题。
			Map<String,String> lTmpHitMap = new HashMap<String,String>();
			lTmpHitMap.putAll(this.mHitMap);
			
			if(!_checkIlleagleProperty(lTmpHitMap)){
				return null;
			}
			_dropAllIllegalFields(lTmpHitMap);
			_translateFieldsName(lTmpHitMap);
			// 如果不包含EventID字段，那么我们直接返回空
			if (!lTmpHitMap.containsKey(UTLogFieldsScheme.EVENTID
					.toString())) {
				return null;
			}
			return lTmpHitMap;
		}

		private static boolean _checkIlleagleProperty(Map<String, String> aHitMap) {
			if (null != aHitMap) {
				
				if(aHitMap.containsKey(null)){
					aHitMap.remove(null);
		        }
		        
		        if(aHitMap.containsKey("")){
		        	aHitMap.remove("");
		        }
				
				if (aHitMap.containsKey(UTLogFieldsScheme.PAGE.toString())) {
					UTMCLogger
					.e(UTMCLogger.LOG_TYPE_DEBUG,
							"checkIlleagleProperty",
							"IlleaglePropertyKey(PAGE) is setted when you call the method setProperty or setProperties ,please use another key to replace it!");
					return false;
				}

				if (aHitMap.containsKey(UTLogFieldsScheme.EVENTID.toString())) {
					UTMCLogger
					.e(UTMCLogger.LOG_TYPE_DEBUG,
							"checkIlleagleProperty",
							"IlleaglePropertyKey(EVENTID) is setted when you call the method setProperty or setProperties ,please use another key to replace it!");
					return false;
				}

				if (aHitMap.containsKey(UTLogFieldsScheme.ARG1.toString())) {
					UTMCLogger
					.e(UTMCLogger.LOG_TYPE_DEBUG,
							"checkIlleagleProperty",
							"IlleaglePropertyKey(ARG1) is setted when you call the method setProperty or setProperties ,please use another key to replace it!");
					return false;
				}

				if (aHitMap.containsKey(UTLogFieldsScheme.ARG2.toString())) {
					UTMCLogger
					.e(UTMCLogger.LOG_TYPE_DEBUG,
							"checkIlleagleProperty",
							"IlleaglePropertyKey(ARG2) is setted when you call the method setProperty or setProperties ,please use another key to replace it!");
					return false;
				}

				if (aHitMap.containsKey(UTLogFieldsScheme.ARG3.toString())) {
					UTMCLogger
					.e(UTMCLogger.LOG_TYPE_DEBUG,
							"checkIlleagleProperty",
							"IlleaglePropertyKey(ARG3) is setted when you call the method setProperty or setProperties ,please use another key to replace it!");
					return false;
				}
			}
			return true;
		}

		// 把fields中的一些key做翻译
		private static void _translateFieldsName(Map<String, String> aHitMap) {
			// 转换key的含义
			if (null != aHitMap) {

				if (aHitMap.containsKey(FIELD_PAGE)) {
					String lValue = aHitMap.get(FIELD_PAGE);
					aHitMap.remove(FIELD_PAGE);
					aHitMap.put(UTLogFieldsScheme.PAGE.toString(), lValue);
				}

				if (aHitMap.containsKey(FIELD_ARG1)) {
					String lValue = aHitMap.get(FIELD_ARG1);
					aHitMap.remove(FIELD_ARG1);
					aHitMap.put(UTLogFieldsScheme.ARG1.toString(), lValue);
				}

				if (aHitMap.containsKey(FIELD_ARG2)) {
					String lValue = aHitMap.get(FIELD_ARG2);
					aHitMap.remove(FIELD_ARG2);
					aHitMap.put(UTLogFieldsScheme.ARG2.toString(), lValue);
				}

				if (aHitMap.containsKey(FIELD_ARG3)) {
					String lValue = aHitMap.get(FIELD_ARG3);
					aHitMap.remove(FIELD_ARG3);
					aHitMap.put(UTLogFieldsScheme.ARG3.toString(), lValue);
				}

				if (aHitMap.containsKey(FIELD_ARGS)) {
					String lValue = aHitMap.get(FIELD_ARGS);
					aHitMap.remove(FIELD_ARGS);
					aHitMap.put(UTLogFieldsScheme.ARGS.toString(), lValue);
				}

				if (aHitMap.containsKey(FIELD_EVENT_ID)) {
					String lValue = aHitMap.get(FIELD_EVENT_ID);
					aHitMap.remove(FIELD_EVENT_ID);
					aHitMap.put(UTLogFieldsScheme.EVENTID.toString(), lValue);
				}

			}
		}

		private static void _dropAllIllegalFields(Map<String, String> aHitMap) {

			if (null != aHitMap) {

				if (aHitMap.containsKey(UTLogFieldsScheme.PAGE.toString())) {
					aHitMap.remove(UTLogFieldsScheme.PAGE.toString());
				}

				if (aHitMap.containsKey(UTLogFieldsScheme.EVENTID.toString())) {
					aHitMap.remove(UTLogFieldsScheme.EVENTID.toString());
				}

				if (aHitMap.containsKey(UTLogFieldsScheme.ARG1.toString())) {
					aHitMap.remove(UTLogFieldsScheme.ARG1.toString());
				}

				if (aHitMap.containsKey(UTLogFieldsScheme.ARG2.toString())) {
					aHitMap.remove(UTLogFieldsScheme.ARG2.toString());
				}

				if (aHitMap.containsKey(UTLogFieldsScheme.ARG3.toString())) {
					aHitMap.remove(UTLogFieldsScheme.ARG3.toString());
				}

				if (aHitMap.containsKey(UTLogFieldsScheme.ARGS.toString())) {
					aHitMap.remove(UTLogFieldsScheme.ARGS.toString());
				}

			}
		}
	}

	/**
	 * 自定义事件打点Builder
	 * @author daichenjie
	 * <pre>
	 * 自定义事件是除页面事件之外，几乎能满足用户的全部需求，可以用它来做abtest，也可以用它来记录各种效果数据。
	 * 自定义包含以下几个部分内容：
	 *    1.事件名称（event_label）
	 *    2.事件从开始到完成消耗的时长
	 *    3.事件所携带的属性
	 *    4.事件对应的页面
	 *    
	 * 例子：
	 *    // 事件名称：play_music
	 *    UTCustomHitBuilder lHitBuilder = new UTHitBuilders.UTCustomHitBuilder(
	 *			"play_music");
	 *    // 设置时长，3分钟，一首歌3分钟合理
	 *    lHitBuilder.setDurationOnEvent(3*60*1000);
	 *    // 设置关联的页面名称：聆听
	 *    lHitBuilder.setEventPage("Listen");
	 *    // 设置属性：类型摇滚
	 *    lHitBuilder.setProperty("type", "rock");
	 *    // 设置属性：歌曲标题
	 *    lHitBuilder.setProperty("title", "wonderful tonight");
     *    // 发送自定义事件打点
	 *    UTAnalytics.getInstance().getDefaultTracker().send(lHitBuilder.build());
	 *
	 * 结果log：
	 *    11-06 16:31:09.248: I/cache_log(14987): UT:省略...一堆||listener||19999||
	 *    play_music||-||180||type=rock,title=wonderful tonight
	 *    
	 *    这个Log里的19999代表自定义事件这个行为
	 *    
	 * </pre>
	 */
	public static class UTCustomHitBuilder extends UTHitBuilder {

		/**
		 * 构造函数
		 * @param aEventLabel 	事件名称，唯一标识一个事件，在事件命名上，需要有区分性，避免不同团队写的名称碰撞
		 */
		public UTCustomHitBuilder(String aEventLabel) {
			if (!UTMCStringUtils.isEmpty(aEventLabel)) {
				super.setProperty(UTHitBuilder.FIELD_ARG1, aEventLabel);
			}
			super.setProperty(UTHitBuilder.FIELD_EVENT_ID, "19999");
			super.setProperty(UTHitBuilder.FIELD_ARG3, "0");
		}

		/**
		 * 设置事件消耗时长（事件完成时间-事件开始时间）
		 * @param aDuration	事件消耗时长（单位：毫秒）
		 * @return
		 */
		public UTCustomHitBuilder setDurationOnEvent(long aDuration) {
			if(aDuration<0){
				aDuration = 0;
			}
			super.setProperty(UTHitBuilder.FIELD_ARG3, "" + aDuration);
			return this;
		}

		/**
		 * 设置自定义事件的页面字段
		 * 
		 * @param aPage 页面名称
		 * @return 
		 */
		public UTCustomHitBuilder setEventPage(String aPage) {
			if (!UTMCStringUtils.isEmpty(aPage)) {
				super.setProperty(UTHitBuilder.FIELD_PAGE, aPage);
			}
			return this;
		}

		/**
		 * 进行URLEncoded转换(arg1不能被转换)
		 */
		@Override
		public Map<String, String> build() {
			Map<String,String> lOMap = super.build();
			if(null != lOMap){
				String lPageName = lOMap.get(UTLogFieldsScheme.PAGE.toString());
				String lEventLabel = lOMap.get(UTLogFieldsScheme.ARG1.toString());
				if(null != lEventLabel){
					lOMap.remove(UTLogFieldsScheme.ARG1.toString());
					lOMap.remove(UTLogFieldsScheme.PAGE.toString());
					Map<String,String> lNMap = UTMCMapUtils.convertToUrlEncodedMap(lOMap);
					lNMap.put(UTLogFieldsScheme.ARG1.toString(), lEventLabel);
					lNMap.put(UTLogFieldsScheme.PAGE.toString(), lPageName);
					return lNMap;
				}
			}
			return lOMap;
		}

	}

	/**
	 * 页面打点Builder
	 * @author daichenjie
	 * <pre>
	 * 页面打点是打点中最重要的事情。
	 * 这个类仅仅提供了页面打点的规范字段，这个类是我们SDK提供给App用来定制页面打点策略（如支持Fragment等）。
	 * 定制的内容包含以下内容：
	 *    1.当前页面页面名称
	 *    2.上一个访问页面
	 *    3.页面停留时长(页面消失时间-页面开始展示时间)
	 *    4.页面属性
	 *    
	 * 例子：
	 *    // 发送页面事件
	 *    UTPageHitBuilder lHitBuilder = new UTPageHitBuilder(lPageName);
	 *    lHitBuilder.setReferPage(lRefPage).setDurationOnPage(lPageStayConsume).setProperties(mPageProperties);
	 *	  UTTracker lTracker = UTAnalytics.getInstance().getDefaultTracker();
	 *    if(null != lTracker){
	 *        lTracker.send(lHitBuilder.build());
	 *    }
	 * </pre>
	 */
	public static class UTPageHitBuilder extends UTHitBuilder {

		/**
		 * 构造函数
		 * @param aPageName 	页面名称，不能为空和null
		 */
		public UTPageHitBuilder(String aPageName) {
			if (!UTMCStringUtils.isEmpty(aPageName)) {
				super.setProperty(UTHitBuilder.FIELD_PAGE, aPageName);
			}
			super.setProperty(UTHitBuilder.FIELD_EVENT_ID, "2001");
			// 默认设置页面时长为0
			super.setProperty(UTHitBuilder.FIELD_ARG3, "0");
		}

		/**
		 * 设置当前页面的的上一个页面名称（上一个引导页面）
		 * @param aReferPage	上一个引导页面
		 * @return
		 */
		public UTPageHitBuilder setReferPage(String aReferPage) {
			if (!UTMCStringUtils.isEmpty(aReferPage)) {
				super.setProperty(UTHitBuilder.FIELD_ARG1, aReferPage);
			}
			return this;
		}

		/**
		 * 设置页面停留时长（消失时间点-开始展示时间点）
		 * @param aDuration		停留时长（若是负数，sdk自动填充0）
		 * @return
		 */
		public UTPageHitBuilder setDurationOnPage(long aDuration) {
			
			// 重复逻辑，避免不必要的错误
			if(aDuration<0){
				aDuration = 0;
			}
			super.setProperty(UTHitBuilder.FIELD_ARG3, "" + aDuration);
			return this;
		}

	}

}
