package com.ut.mini;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Queue;

import android.app.Activity;
import android.net.Uri;
import android.os.SystemClock;
import android.text.TextUtils;
import android.util.Log;

import com.ut.mini.UTHitBuilders.UTPageHitBuilder;
import com.ut.mini.base.UTMIVariables;
import com.ut.mini.core.UTMCVariables;
import com.ut.mini.core.onlineconf.UTMCTPKBiz;
import com.ut.mini.log.UTMCLogger;
import com.ut.mini.utils.UTMCStringUtils;

/**
 * 页面打点辅助类
 * 
 * @author daichenjie
 * 
 *         <pre>
 * 
 * 页面采集包含以下几个元素：
 *    1.采集当前页面名称
 *    2.之前访问的页面（refer page）
 *    3.页面展示的时长
 *    4.页面的一些特性属性
 * 
 * 页面采集也间接的完成了，app在前台停留时长的这个指标及app启动次数。
 * 
 * 这个类的目标都在是让app开发者更少的使用UTPageHitBuilder，来完成页面的(展示、消失)打点，提高数据质量。
 * 
 * 说明：
 *    1.UTPageHitHelper主要完成Activity页面的采集，不包含Fragment，如果需要支持Fragment的，可以扩展当前类
 * 或者自己完成全部的逻辑，通过UTPageHitBuilder来完成规范要求的页面打点工作。
 *    2.UTPageHitHelper支持4.0及以上的版本的android系统，可以自动完成全部的工作，开发者不需要做任何activity
 * 的埋点，唯一的成本是，需要设置UTAnalytics.getInstance().setAppApplicationInstance(instance);
 *       
 *    那么4.0以下怎么办呢？
 *       1.如果用户的应用最少支持4.0到以上，那么不需要care，只需要按上述说明去做就可以
 *       2.如果用户的应用需要支持4.0以下的市场，那么应用方最好有一个activity的基类，基类中的onReasume、onPause
 *       按规范完成页面的打点。
 *       
 * 怎么用呢？
 *    1.4.0到以上
 *       UTAnalytics.getInstance().setAppApplicationInstance(instance)
 *    2.4.0以下
 *       activity的onReasume
 *           pageAppear(Activity aActivity)
 *       activity的onPause
 *           pageDisAppear(Activity aActivity)
 *           
 * updatePageProperties是一个什么玩意儿？
 *    举个例子：我们要开发一个购物的app，购物的app的宝贝展示页面，我们可能想要知道，宝贝展示页面，都展示了哪些商品呢？
 *            可以简单理解为，我们可以对采集的页面（GoodsDetails页面）记录上增加一个宝贝属性就可以，如item_id=xxxxxx
 *            我们可以通过sql，选择GoodsDetails页面就可以对其中的属性item_id做分析，就可以满足我们的需求。
 *            
 *    使用方法：
 *       // 这句话必须要在一个页面的onPause之前调用都可以
 *       Map<String, String> lMap = new HashMap<String, String>();
 *       lMap.put("item_id", "xxxxxx");
 *       UTPageHitHelper.getInstance().updatePageProperties(lMap);
 *       
 *       结果Log：
 *          11-06 16:10:32.088: I/cache_log(10879): UT:省略...一堆||Test||2001||-||-||3191||item_id=xxxxxx
 *       
 *          这里：2001是页面事件的ID，3191是页面展示的时长，item_id=xxxxxx就是属性
 *       
 *       特别强调：4.0以下updatePageProperties需要配合pageAppear和pageDisAppear使用
 * 
 * </pre>
 */
public class UTPageHitHelper {

	public static class UTPageEventObject {
		private Map<String, String> mPageProperties = new HashMap<String, String>();
		private long mPageStayTimstamp = 0l;
		private Uri mPageUrl = null;
		private String mPageName = null;
		private String mRefPage = null;
		private UTPageStatus mPageStatus = null;
		private boolean mIsPageAppearCalled = false;
		private boolean mIsSkipPage = false;
		private boolean mIsH5Called = false;
		private String mCacheKey = null;
		
		public void setCacheKey(String aCacheKey){
			mCacheKey = aCacheKey;
		}
		
		public String getCacheKey(){
			return mCacheKey;
		}

		public void resetPropertiesWithoutSkipFlagAndH5Flag() {
			mPageProperties = new HashMap<String, String>();
			mPageStayTimstamp = 0l;
			mPageUrl = null;
			mPageName = null;
			mRefPage = null;
			if (!(null != mPageStatus && mPageStatus == UTPageStatus.UT_H5_IN_WebView)) {
				mPageStatus = null;
			}
			mIsPageAppearCalled = false;
			mIsH5Called = false;
		}

		public boolean isH5Called() {
			return mIsH5Called;
		}

		public void setH5Called() {
			mIsH5Called = true;
		}

		public void setToSkipPage() {
			mIsSkipPage = true;
		}

		public boolean isSkipPage() {
			return mIsSkipPage;
		}

		public void setPageAppearCalled() {
			mIsPageAppearCalled = true;
		}

		public boolean isPageAppearCalled() {
			return mIsPageAppearCalled;
		}

		public void setPageStatus(UTPageStatus aPageStatus) {
			mPageStatus = aPageStatus;
		}

		public UTPageStatus getPageStatus() {
			return mPageStatus;
		}

		public Map<String, String> getPageProperties() {
			return mPageProperties;
		}

		public void setPageProperties(Map<String, String> aPageProperties) {
			this.mPageProperties = aPageProperties;
		}

		public long getPageStayTimstamp() {
			return mPageStayTimstamp;
		}

		public void setPageStayTimstamp(long aPageStayTimstamp) {
			this.mPageStayTimstamp = aPageStayTimstamp;
		}

		public Uri getPageUrl() {
			return mPageUrl;
		}

		public void setPageUrl(Uri aPageUrl) {
			this.mPageUrl = aPageUrl;
		}

		public void setPageName(String aPageName) {
			mPageName = aPageName;
		}

		public String getPageName() {
			return mPageName;
		}

		public void setRefPage(String aRefPage) {
			this.mRefPage = aRefPage;
		}

		public String getRefPage() {
			return this.mRefPage;
		}
	}

	private boolean mIsTurnOff = false;
	private static UTPageHitHelper s_instance = new UTPageHitHelper();
	private Map<String, String> mPageProperties = new HashMap<String, String>();
	private Map<String, UTPageEventObject> mPageEventObjects = new HashMap<String, UTPageEventObject>();
	private String mCurrentPageCacheKey = null;
	private Map<String, String> mNextPageProperties = new HashMap<String, String>();
	private String mCurPage = null;
	private Queue<UTPageEventObject> mSkipClearPageObjectList = new LinkedList<UTPageEventObject>();
	private final static int MAX_SKIP_CLEAR_PAGE_OBJECT_CACHE_CAPACITY = 100;
	private Map<Object,String> mSPMObjectList = new HashMap<Object,String>();

	/**
	 * 获取UTPageHitHelper实例
	 * 
	 * @return
	 */
	public static UTPageHitHelper getInstance() {
		return s_instance;
	}

	synchronized Map<String, String> getAndResetNextPageProperties() {
		if (null != mNextPageProperties && mNextPageProperties.size() > 0) {
			Map<String, String> lHashMap = new HashMap<String, String>();
			lHashMap.putAll(mNextPageProperties);
			mNextPageProperties.clear();
			return lHashMap;
		}
		return null;
	}

	/**
	 * 因为程序中对于skip的和H5的page对象，我们都是不清除的，所以，我们需要通过cache的形式
	 * 对于历史比较早的对象去清理掉。
	 * @param aPageObject
	 * @param aPageEventObject
	 */
	synchronized void _releaseSkipFlagAndH5FlagPageObject(
			UTPageEventObject aPageEventObject) {
		aPageEventObject.resetPropertiesWithoutSkipFlagAndH5Flag();
		if(!mSkipClearPageObjectList.contains(aPageEventObject)){
			mSkipClearPageObjectList.add(aPageEventObject);
		}
		if(mSkipClearPageObjectList.size()>MAX_SKIP_CLEAR_PAGE_OBJECT_CACHE_CAPACITY * 2){
			for(int i=0;i<MAX_SKIP_CLEAR_PAGE_OBJECT_CACHE_CAPACITY;i++){
				UTPageEventObject lPageEventObject = mSkipClearPageObjectList.poll();
				if(null != lPageEventObject){
					if(mPageEventObjects.containsKey(lPageEventObject.getCacheKey())){
						mPageEventObjects.remove(lPageEventObject.getCacheKey());
					}
				}
			}
		}
	}
	
	/**
	 * 测试代码，仅仅用于测试skip&h5 pageEventObject的缓存
	 */
//	public void testreleaseH5andskipmethod(){
//		for(int i=1;i<2000;i++){
//			UTPageEventObject lUTPageEventObj = new UTPageEventObject();
//			lUTPageEventObj.setCacheKey("cache_key"+i);
//			mPageEventObjects.put("cache_key"+i, lUTPageEventObj);
//			_releaseSkipFlagAndH5FlagPageObject(lUTPageEventObj);
//		}
//	}

	/**
	 * 关闭页面自动采集功能 {@link com.ut.mini.UTAnalytics}
	 */
	@Deprecated
	public synchronized void turnOffAutoPageTrack() {
		mIsTurnOff = true;
	}

	public String getCurrentPageName() {
		return mCurPage;
	}

	void pageAppearByAuto(Activity aActivity) {
		if (mIsTurnOff) {
			return;
		}
		pageAppear(aActivity);
	}

	private String _getPageEventObjectCacheKey(Object aPageObject) {
		String lPageName = null;
		if (aPageObject instanceof String) {
			lPageName = (String) aPageObject;
		} else {
			lPageName = aPageObject.getClass().getSimpleName();
		}
		int lHashCode = aPageObject.hashCode();
		return lPageName + lHashCode;
	}

	synchronized boolean isH52001(Object aPageObject) {
		if (null != aPageObject) {
			UTPageEventObject lPEObject = _getOrNewAUTPageEventObject(aPageObject);
			if (null != lPEObject.getPageStatus()) {
				if (lPEObject.getPageStatus() == UTPageStatus.UT_H5_IN_WebView) {
					return true;
				}
			}
		}
		return false;
	}

	synchronized void setH5Called(Object aPageObject) {
		if (null != aPageObject) {
			UTPageEventObject lPEObject = _getOrNewAUTPageEventObject(aPageObject);
			if (null != lPEObject.getPageStatus()) {
				lPEObject.setH5Called();
			}
		}
	}

	private synchronized UTPageEventObject _getOrNewAUTPageEventObject(
			Object aPageObject) {
		String lCacheKey = _getPageEventObjectCacheKey(aPageObject);
		if (mPageEventObjects.containsKey(lCacheKey)) {
			return mPageEventObjects.get(lCacheKey);
		} else {
			UTPageEventObject lPageEventObject = new UTPageEventObject();
			mPageEventObjects.put(lCacheKey, lPageEventObject);
			lPageEventObject.setCacheKey(lCacheKey);
			return lPageEventObject;
		}
	}

	private synchronized void _putUTPageEventObjectToCache(String aCacheKey,
			UTPageEventObject aPageObject) {
		mPageEventObjects.put(aCacheKey, aPageObject);
	}

	private synchronized void _clearUTPageEventObjectCache(UTPageEventObject aPageEventObject) {
		if(mPageEventObjects.containsKey(aPageEventObject.getCacheKey())){
			mPageEventObjects.remove(aPageEventObject.getCacheKey());
		}
	}

	private synchronized void _removeUTPageEventObject(Object aPageObject) {
		String lCacheKey = _getPageEventObjectCacheKey(aPageObject);
		if (mPageEventObjects.containsKey(lCacheKey)) {
			mPageEventObjects.remove(lCacheKey);
		}
	}

	/**
	 * 页面开始展现
	 * 
	 * @param aPageObject
	 *            {@link com.ut.mini.UTTracker}
	 */
	@Deprecated
	public synchronized void pageAppear(Object aPageObject) {
		pageAppear(aPageObject, null, false);
	}

	synchronized void pageAppear(Object aPageObject, String aCustomPageName,
			boolean aIsDonotSkipFlag) {
		// 如果从后台来，那么我们设置应用最新的使用时长为0
		if (null != aPageObject) {
			// 判断是否重复进入了同一个页面
			String lPageCacheKey = this
					._getPageEventObjectCacheKey(aPageObject);
			if (null != lPageCacheKey
					&& lPageCacheKey.equals(this.mCurrentPageCacheKey)) {
				return;
			}

			if (null != this.mCurrentPageCacheKey) {
				UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG, "lost 2001",
						"Last page requires leave(" + mCurrentPageCacheKey
								+ ").");
			}

			UTPageEventObject lPEObject = _getOrNewAUTPageEventObject(aPageObject);
			if (!aIsDonotSkipFlag && lPEObject.isSkipPage()) {
				UTMCLogger.i(UTMCLogger.LOG_TYPE_DEBUG,
						"skip page[pageAppear]", "page name:"
								+ aPageObject.getClass().getSimpleName());
				return;
			}

			String lH5Url = UTMIVariables.getInstance().getH5Url();
			if (lH5Url != null) {
				// Caused by: java.lang.UnsupportedOperationException: This
				// isn't a hierarchical URI.
				try {
					Uri lUrl = Uri.parse(lH5Url);
					String lSpm = lUrl.getQueryParameter("spm");
					mPageProperties.put("spm", lSpm);
				} catch (Throwable e) {
					e.printStackTrace();
				}
				UTMIVariables.getInstance().setH5Url(null);// H5->Native 只取一跳
															// 取完清空
			}

			String lCurPage = _getPageName(aPageObject);

			if (!UTMCStringUtils.isEmpty(aCustomPageName)) {
				lCurPage = aCustomPageName;
			}
			if (!UTMCStringUtils.isEmpty(lPEObject.getPageName())) {
				lCurPage = lPEObject.getPageName();
			}
			mCurPage = lCurPage;
			lPEObject.setPageName(lCurPage);
			lPEObject.setPageStayTimstamp(SystemClock.elapsedRealtime());
			lPEObject.setRefPage(UTMIVariables.getInstance().getRefPage());
			lPEObject.setPageAppearCalled();
			// 对于 NextPageProperties我们只在Appear中消费
			if (null != mNextPageProperties) {
				Map<String, String> lPageProperties = lPEObject
						.getPageProperties();
				if (null == lPageProperties) {
					lPEObject.setPageProperties(this.mNextPageProperties);
				} else {
					Map<String, String> lNewPageProperties = new HashMap<String, String>();
					lNewPageProperties.putAll(lPageProperties);
					lNewPageProperties.putAll(mNextPageProperties);
					lPEObject.setPageProperties(lNewPageProperties);
				}
			}
			mNextPageProperties = null;
			mCurrentPageCacheKey = this
					._getPageEventObjectCacheKey(aPageObject);

			// 清理当前的UTPageEventObject的缓存
			_clearUTPageEventObjectCache(lPEObject);
			// 再重新把这个key填充进去,防止key太多溢出
			_putUTPageEventObjectToCache(
					_getPageEventObjectCacheKey(aPageObject), lPEObject);
		} else {
			UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG, "pageAppear",
					"The page object should not be null");
		}
	}

	synchronized void pageAppear(Object aPageObject, String aCustomPageName) {
		pageAppear(aPageObject, aCustomPageName, false);
	}

	/**
	 * 更新页面属性，这些属性可以在数据表的args字段中提取出来
	 * 
	 * @param aProperties
	 *            属性 {@link com.ut.mini.UTTracker}
	 */
	@Deprecated
	public synchronized void updatePageProperties(
			Map<String, String> aProperties) {
		if (null != aProperties) {
			mPageProperties.putAll(aProperties);
		}
	}

	/**
	 * 更新页面参数
	 * 
	 * @param aPageObject
	 * @param aProperties
	 */
	synchronized void updatePageProperties(Object aPageObject,
			Map<String, String> aProperties) {
		if (null == aPageObject || null == aProperties
				|| aProperties.size() == 0) {
			UTMCLogger
					.e(UTMCLogger.LOG_TYPE_DEBUG,
							"updatePageProperties",
							"failed to update project, parameters should not be null and the map should not be empty");
			return;
		}
		Map<String, String> lAMap = new HashMap<String, String>();
		lAMap.putAll(aProperties);
		UTPageEventObject lPEObject = _getOrNewAUTPageEventObject(aPageObject);
		Map<String, String> lPageProperties = lPEObject.getPageProperties();
		if (null == lPageProperties) {
			lPEObject.setPageProperties(lAMap);
		} else {
			Map<String, String> lNewPageProperties = new HashMap<String, String>();
			lNewPageProperties.putAll(lPageProperties);
			lNewPageProperties.putAll(lAMap);
			lPEObject.setPageProperties(lNewPageProperties);
		}
	}

	/**
	 * 更新页面名称
	 * 
	 * @param aPageObject
	 *            页面对象
	 * @param aPageName
	 *            页面名称
	 */
	synchronized void updatePageName(Object aPageObject, String aPageName) {
		if (null == aPageObject || UTMCStringUtils.isEmpty(aPageName)) {
			return;
		}
		UTPageEventObject lPEObject = _getOrNewAUTPageEventObject(aPageObject);
		lPEObject.setPageName(aPageName);
		mCurPage = aPageName;
	}

	synchronized void updatePageUrl(Object aPageObject, Uri aUrl) {
		if (null == aPageObject || null == aUrl) {
			return;
		}
		Log.i("url", "url" + aUrl.toString());
		UTPageEventObject lPEObject = _getOrNewAUTPageEventObject(aPageObject);
		lPEObject.setPageUrl(aUrl);
	}

	synchronized void updatePageStatus(Object aPageObject,
			UTPageStatus aPageStatus) {
		if (null == aPageObject || null == aPageStatus) {
			return;
		}
		UTPageEventObject lPEObject = _getOrNewAUTPageEventObject(aPageObject);
		lPEObject.setPageStatus(aPageStatus);
	}

	synchronized void updateNextPageProperties(Map<String, String> aProperties) {
		if (null != aProperties) {
			Map<String, String> lAMap = new HashMap<String, String>();
			lAMap.putAll(aProperties);
			mNextPageProperties = lAMap;
		}
	}

	void pageDisAppearByAuto(Activity aActivity) {
		if (mIsTurnOff) {
			return;
		}
		pageDisAppear(aActivity);
	}

	synchronized void skipPage(Object aPageObject) {
		if (null == aPageObject) {
			return;
		}
		UTPageEventObject lPageEventObject = this
				._getOrNewAUTPageEventObject(aPageObject);
		lPageEventObject.setToSkipPage();
	}

	/**
	 * 页面开始消失
	 * 
	 * @param aPageObject
	 *            {@link com.ut.mini.UTTracker}
	 */
	@Deprecated
	public synchronized void pageDisAppear(Object aPageObject) {
		if (null != aPageObject) {

			// 如果当前页面已经离开过一次了，那么不允许再次离开
			if (null == mCurrentPageCacheKey) {
				// UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG,
				// "pageDisAppear","UT has already recorded the page disappear event on this page");
				return;
			}

			UTPageEventObject lPageEventObject = this
					._getOrNewAUTPageEventObject(aPageObject);
			// 如果当前页面不是aplus上认为的2001页面
			if (lPageEventObject.isPageAppearCalled()) {

				// 如果是H5页面直接离开
				if (null != lPageEventObject.getPageStatus()
						&& UTPageStatus.UT_H5_IN_WebView == lPageEventObject
								.getPageStatus()) {
					// 如果h5的2001调用过
					if (lPageEventObject.isH5Called()) {
						// 如果是页面状态标记为H5，那么我们也不清理标记
						_releaseSkipFlagAndH5FlagPageObject(lPageEventObject);
						return;
					}
				}

				long lPageStayConsume = SystemClock.elapsedRealtime()
						- lPageEventObject.getPageStayTimstamp();

				// 假设外部没有塞入pageUrl，如果是activity，那么我们需要去塞入
				if (null == lPageEventObject.getPageUrl()) {
					if (aPageObject instanceof Activity) {
						lPageEventObject.setPageUrl(((Activity) aPageObject)
								.getIntent().getData());
					}
				}

				String lPageName = lPageEventObject.getPageName();

				String lRefPage = lPageEventObject.getRefPage();
				// 计算，引用页面
				/*
				 * String lRefPage = "-"; if (null != mRefPage &&
				 * mRefPage.length() > 0) { lRefPage = mRefPage; }
				 */
				// String lRefPage = UTMIVariables.getInstance().getmRefPage();
				if (null == lRefPage || lRefPage.length() == 0) {
					lRefPage = "-";
				}

				Map<String, String> lPageProperties = mPageProperties;

				if (null == lPageProperties) {
					lPageProperties = new HashMap<String, String>();
				}

				if (null != lPageEventObject.getPageProperties()) {
					lPageProperties
							.putAll(lPageEventObject.getPageProperties());
				}

				if (aPageObject instanceof IUTPageTrack) {

					IUTPageTrack lUTActivity = (IUTPageTrack) aPageObject;

					// 如果基类中有设置pagerefer，那么我们直接提取
					String lARefPage = lUTActivity.getReferPage();
					if (!UTMCStringUtils.isEmpty(lARefPage)) {
						lRefPage = lARefPage;
					}

					Map<String, String> lPageProperties2 = lUTActivity
							.getPageProperties();

					if (null != lPageProperties2 && lPageProperties2.size() > 0) {
						this.mPageProperties.putAll(lPageProperties2);
						lPageProperties = this.mPageProperties;
					}

					// 判断页面名
					String lUTPageName = lUTActivity.getPageName();
					if (!UTMCStringUtils.isEmpty(lUTPageName)) {
						lPageName = lUTPageName;
					}

				}

				// 处理额外的map
				Uri lUrl = lPageEventObject.getPageUrl();
				if (null != lUrl) {
					// Caused by: java.lang.UnsupportedOperationException: This
					// isn't a hierarchical URI.
					try {
						Map<String, String> lEMap = new HashMap<String, String>();
						String lSPM = lUrl.getQueryParameter("spm");
						if (UTMCStringUtils.isEmpty(lSPM)) {
							try {
								lUrl = Uri.parse(URLDecoder.decode(
										lUrl.toString(), "UTF-8"));
								lSPM = lUrl.getQueryParameter("spm");
							} catch (UnsupportedEncodingException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}
						}
						if (!UTMCStringUtils.isEmpty(lSPM)) {
							boolean mIsSPMSkip = false;
							// 如果之前已经获取过，并且相同的，那么直接跳过。。。
							if(mSPMObjectList.containsKey(aPageObject)){
								if(lSPM.equals(mSPMObjectList.get(aPageObject))){
									mIsSPMSkip = true;
								}
							}
							if(!mIsSPMSkip){
								lEMap.put("spm", lSPM);
								this.mSPMObjectList.put(aPageObject, lSPM);
							}
						}
						String lSCM = lUrl.getQueryParameter("scm");
						if (!UTMCStringUtils.isEmpty(lSCM)) {
							lEMap.put("scm", lSCM);
						}

						String lOutsideTTID = _getOutsideTTID(lUrl);
						if (!UTMCStringUtils.isEmpty(lOutsideTTID)) {
							UTMCVariables.getInstance().setOutsideTTID(
									lOutsideTTID);
						}

						if (lEMap.size() > 0) {
							lPageProperties.putAll(lEMap);
						}
					} catch (Throwable e) {
						e.printStackTrace();
					}
				}

				try {
					// 添加tpk逻辑
					String lTPKString = UTMCTPKBiz.getInstance().getTpkString(
							lPageEventObject.getPageUrl(), lPageProperties);

					if (!UTMCStringUtils.isEmpty(lTPKString)) {
						lPageProperties.put("_tpk", lTPKString);
					}
				} catch (Exception e) {
					e.printStackTrace();
				}

				// 发送页面事件
				UTPageHitBuilder lHitBuilder = new UTPageHitBuilder(lPageName);
				/*
				 * lPageName:当前页面 lRefPage：前一个出现页面
				 * lPageStayConsume:页面展现到不展现消耗的时长
				 */
				// 页面：控件引用，这个怎么去处理....
				lHitBuilder.setReferPage(lRefPage)
						.setDurationOnPage(lPageStayConsume)
						.setProperties(lPageProperties);

				// 设置当前页面为下一个页面的refpage

				// mRefPage = lPageName;
				UTMIVariables.getInstance().setRefPage(lPageName);

				UTTracker lTracker = UTAnalytics.getInstance()
						.getDefaultTracker();
				if (null != lTracker) {
					lTracker.send(lHitBuilder.build());
				} else {
					UTMCLogger
							.e(UTMCLogger.LOG_TYPE_DEBUG,
									"Record page event error",
									"Fatal Error,must call setRequestAuthentication method first.");
				}

			} else {
				UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG, "UT",
						"Please call pageAppear first("
								+ _getPageName(aPageObject) + ").");
			}

			// 置空：属性列表
			mPageProperties = new HashMap<String, String>();
			// 清除页面对象
			if (lPageEventObject.isSkipPage()) {
				_releaseSkipFlagAndH5FlagPageObject(lPageEventObject);
			} else if (null != lPageEventObject.getPageStatus()
					&& UTPageStatus.UT_H5_IN_WebView == lPageEventObject
							.getPageStatus()) {
				// 如果页面是H5的话，那么我们只清理属性
				_releaseSkipFlagAndH5FlagPageObject(lPageEventObject);
			} else {
				_removeUTPageEventObject(aPageObject);
			}
			mCurrentPageCacheKey = null;
			mCurPage = null;
		} else {
			UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG, "pageDisAppear",
					"The page object should not be null");
		}
	}

	private static String _getOutsideTTID(Uri pUri) {
		if (null != pUri) {
			List<String> lResult = pUri.getQueryParameters("ttid");
			if (null != lResult) {
				for (String str : lResult) {
					if (!str.contains("@")) {
						if (str.contains("%40")) {
							continue;
						}
						return str;
					}
				}
			}
		}
		return null;
	}

	private static String _getPageName(Object aPageObject) {
		String lPageName = aPageObject.getClass().getSimpleName();
		if (null != lPageName && lPageName.toLowerCase().endsWith("activity")) {
			lPageName = lPageName.substring(0, lPageName.length() - 8);
			return lPageName;
		}
		return lPageName;
	}

}
