package com.ut.mini;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import android.net.Uri;

import com.ut.mini.base.UTConstants;
import com.ut.mini.base.UTLogFieldsScheme;
import com.ut.mini.base.UTMCLogFields;
import com.ut.mini.base.UTMIVariables;
import com.ut.mini.core.UTMCLogTransferMain;
import com.ut.mini.core.onlineconf.UTMCTPKBiz;
import com.ut.mini.core.onlineconf.UTTPKItem;
import com.ut.mini.log.UTMCLogger;
import com.ut.mini.utils.UTMCStringUtils;

/**
 * UT统计发送打点类
 * @author daichenjie
 * <pre>
 * 这个类主要完成把打点数据发送出去(send)。
 * 
 * 除了发送，还有如下几个公开方法：getGlobalProperty(String aKey)、removeGlobalProperty(String aKey)、
 * 							setGlobalProperty(String aKey, String aValue)
 * 那么这几个方法解决什么问题呢？
 *    先看一个例子：
 *       UTCustomHitBuilder lHitBuilder = new UTHitBuilders.UTCustomHitBuilder(
 *				"play_music");
 *       lHitBuilder.setProperty("type", "rock");
 *       lHitBuilder.setProperty("title", "wonderful tonight");
 *       //通过defaultTracker发送打点
 *       UTAnalytics.getInstance().getDefaultTracker().send(lHitBuilder.build());
 *       
 *       这里可以看出，Tracker发送的数据是lHitBuilder.build()，也就是一个Map<String,String>结构体。
 *       这几个方法就和这个Map结构体有关系：Global的含义就是这个特定Tracker每次send数据的时候，给build的Map都加上这些属性对
 *       
 *    怎么使用？
 *       //这里给DefaultTracker设定一个GlobalProperty
 *       UTAnalytics.getInstance().getDefaultTracker().setGlobalProperty("GlobalKey", "GlobalValue");
 *       //下面发送一个自定义事件打点，测试效果
 *       UTCustomHitBuilder lHitBuilder = new UTHitBuilders.UTCustomHitBuilder("TestGlobalKey");
 *       //发送数据
 *       UTAnalytics.getInstance().getDefaultTracker().send(lHitBuilder.build());
 *       
 *       结果如何？
 *          11-06 14:22:35.801: I/cache_log(455): UT:UT:省略...一堆||UT||19999
 *          ||TestGlobalKey||-||0||GlobalKey=GlobalValue
 *          
 *          这个自定义事件的末尾加了一个GlobalKey=GlobalValue的内容，就是这么神奇！
 *          
 *       getGlobalProperty和removeGlobalProperty：我想你懂的！
 * 
 * </pre>
 */
public class UTTracker {

	private String mTrackerId = null;
	private Map<String, String> mArgsMap = new HashMap<String, String>();
	private static Pattern s_p = Pattern.compile("(\\|\\||[\t\r\n])+");

	void setTrackId(String aTrackerId) {
		mTrackerId = aTrackerId;
	}

	/**
	 * 这个全局参数，这个Tracker中所有的打点，都会在args字段中带上这个属性。
	 * @param aKey		key，不能为空和null
	 * @param aValue	value，不能为null
	 */
	public synchronized void setGlobalProperty(String aKey, String aValue) {
		if (!UTMCStringUtils.isEmpty(aKey) && null != aValue) {
			mArgsMap.put(aKey, aValue);
		}else{
			UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG, "setGlobalProperty", "key is null or key is empty or value is null,please check it!");
		}
	}

	/**
	 * 获取全局参数
	 * @param aKey
	 * @return
	 */
	public synchronized String getGlobalProperty(String aKey) {
		if (null != aKey) {
			return mArgsMap.get(aKey);
		}
		return null;
	}

	/**
	 * 移除全局参数
	 * @param aKey
	 */
	public synchronized void removeGlobalProperty(String aKey) {
		if (null != aKey) {
			if (mArgsMap.containsKey(aKey)) {
				mArgsMap.remove(aKey);
			}
		}
	}
	
	/**
	 * 去除换行符以及双竖线
	 * 
	 * @param pStr
	 * @return
	 */
	private static String _getStringNoBlankAndDLine(String pStr) {
		if (pStr != null && !"".equals(pStr)) {

			Matcher m = s_p.matcher(pStr);
			String strNoBlank = m.replaceAll("");
			
			return strNoBlank;

		}
		return pStr;
	}

	private static String _checkField(String pField) {
		pField = _getStringNoBlankAndDLine(pField);
		return pField;
	}
	
	private Map<String, String> _checkMapFields(Map<String, String> aLogMap){
		if(null != aLogMap){
			Map<String,String> lNewMap = new HashMap<String, String>();
			Iterator<String> lKeyItor = aLogMap.keySet().iterator();
			if(null != lKeyItor){
				while(lKeyItor.hasNext()){
					String lKey = lKeyItor.next();
					if(null != lKey){
						String lValue = _checkField(aLogMap.get(lKey));
						lNewMap.put(lKey, lValue);
					}
				}
			}
			return lNewMap;
		}
		return null;
	}
	
	/**
	 * 发送打点
	 * @param aLogMap	打点的元数据，需要通过UTHitBuilder的子类的build方法去生成
	 */
	public void send(Map<String, String> aLogMap) {
		// 检查run的条件是否具备
		if (null != aLogMap) {
			Map<String, String> lNewMap = new HashMap<String, String>();
			lNewMap.putAll(mArgsMap);
			lNewMap.putAll(aLogMap);
			
			// 检查 每个字段的内容是否合法
			lNewMap = _checkMapFields(lNewMap);
			
			if (!UTMCStringUtils.isEmpty(mTrackerId)) {
				lNewMap.put(UTFields.TRACK_ID, mTrackerId);
			}
			
			if(UTMIVariables.getInstance().isAliyunOSPlatform()){
				lNewMap.put(UTMCLogFields.ALIYUN_PLATFORM_FLAG.toString(), "yes");
			}
			
			// 如果LogMap有一个标记，标识允许所有的字段都不做key合法性检查的话，那么我们直接跳过，否则执行检查。
			if(!aLogMap.containsKey(UTConstants.PrivateLogFields.FLAG_USE_ALL_MAP_FIELDS)){
				// 先抛弃不合理的 Log key
				_dropAllIllegalKey(lNewMap);
			}else{
				// 删除这个 特权的key，不遗留在args中
				lNewMap.remove(UTConstants.PrivateLogFields.FLAG_USE_ALL_MAP_FIELDS);
			}
			
			// 把fields中的一些key做翻译
			_translateFieldsName(lNewMap);
			_fillReserve1Fields(lNewMap);
			_fillReservesFields(lNewMap);
			
			//Fix:取消塞SDK_VERSION，使用打包系统定制的UTMCBuildInfo对象
			
			// 发送Log
			UTMCLogTransferMain.getInstance().transferLog(lNewMap);
		}
	}

	/**
	 * 移除了非法的key
	 * 
	 * @param aLogMap
	 */
	private static void _dropAllIllegalKey(Map<String, String> aLogMap) {
		if (null != aLogMap) {
			if (aLogMap.containsKey(UTLogFieldsScheme.IMEI.toString())) {
				aLogMap.remove(UTLogFieldsScheme.IMEI.toString());
			}

			if (aLogMap.containsKey(UTLogFieldsScheme.IMSI.toString())) {
				aLogMap.remove(UTLogFieldsScheme.IMSI.toString());
			}

			if (aLogMap.containsKey(UTLogFieldsScheme.CARRIER.toString())) {
				aLogMap.remove(UTLogFieldsScheme.CARRIER.toString());
			}

			if (aLogMap.containsKey(UTLogFieldsScheme.ACCESS.toString())) {
				aLogMap.remove(UTLogFieldsScheme.ACCESS.toString());
			}

			if (aLogMap.containsKey(UTLogFieldsScheme.ACCESS_SUBTYPE
					.toString())) {
				aLogMap.remove(UTLogFieldsScheme.ACCESS_SUBTYPE.toString());
			}

			if (aLogMap.containsKey(UTLogFieldsScheme.CHANNEL.toString())) {
				aLogMap.remove(UTLogFieldsScheme.CHANNEL.toString());
			}

			if (aLogMap.containsKey(UTLogFieldsScheme.LL_USERNICK.toString())) {
				aLogMap.remove(UTLogFieldsScheme.LL_USERNICK.toString());
			}

			if (aLogMap.containsKey(UTLogFieldsScheme.USERNICK.toString())) {
				aLogMap.remove(UTLogFieldsScheme.USERNICK.toString());
			}

			if (aLogMap.containsKey(UTLogFieldsScheme.LL_USERID.toString())) {
				aLogMap.remove(UTLogFieldsScheme.LL_USERID.toString());
			}

			if (aLogMap.containsKey(UTLogFieldsScheme.USERID.toString())) {
				aLogMap.remove(UTLogFieldsScheme.USERID.toString());
			}

			if (aLogMap.containsKey(UTLogFieldsScheme.SDKVERSION.toString())) {
				aLogMap.remove(UTLogFieldsScheme.SDKVERSION.toString());
			}

			if (aLogMap.containsKey(UTLogFieldsScheme.START_SESSION_TIMESTAMP
					.toString())) {
				aLogMap.remove(UTLogFieldsScheme.START_SESSION_TIMESTAMP
						.toString());
			}

			if (aLogMap.containsKey(UTLogFieldsScheme.UTDID.toString())) {
				aLogMap.remove(UTLogFieldsScheme.UTDID.toString());
			}

			if (aLogMap.containsKey(UTLogFieldsScheme.SDKTYPE.toString())) {
				aLogMap.remove(UTLogFieldsScheme.SDKTYPE.toString());
			}

			if (aLogMap.containsKey(UTLogFieldsScheme.RESERVE2.toString())) {
				aLogMap.remove(UTLogFieldsScheme.RESERVE2.toString());
			}

			if (aLogMap.containsKey(UTLogFieldsScheme.RESERVE3.toString())) {
				aLogMap.remove(UTLogFieldsScheme.RESERVE3.toString());
			}

			if (aLogMap.containsKey(UTLogFieldsScheme.RESERVE4.toString())) {
				aLogMap.remove(UTLogFieldsScheme.RESERVE4.toString());
			}

			if (aLogMap.containsKey(UTLogFieldsScheme.RESERVE5.toString())) {
				aLogMap.remove(UTLogFieldsScheme.RESERVE5.toString());
			}

			if (aLogMap.containsKey(UTLogFieldsScheme.RESERVES.toString())) {
				aLogMap.remove(UTLogFieldsScheme.RESERVES.toString());
			}

			if (aLogMap.containsKey(UTLogFieldsScheme.RECORD_TIMESTAMP
					.toString())) {
				aLogMap.remove(UTLogFieldsScheme.RECORD_TIMESTAMP.toString());
			}

		}
	}

	// 把fields中的一些key做翻译
	private static void _translateFieldsName(Map<String, String> aLogMap) {
		// 转换key的含义
		if (null != aLogMap) {
			// OS
			if (aLogMap.containsKey(UTFields.OS)) {
				String lValue = aLogMap.get(UTFields.OS);
				aLogMap.remove(UTFields.OS);
				aLogMap.put(UTLogFieldsScheme.OS.toString(), lValue);
			}

			// OS——VERSION
			if (aLogMap.containsKey(UTFields.OS_VERSION)) {
				String lValue = aLogMap.get(UTFields.OS_VERSION);
				aLogMap.remove(UTFields.OS_VERSION);
				aLogMap.put(UTLogFieldsScheme.OSVERSION.toString(), lValue);
			}

		}
	}
	
	/**
	 * 填充SDKType到原有的reserve1字段
	 * @param aLogMap
	 * Reserve1这个字段从后期就修正为SDKTYPE
	 */
	private static void _fillReserve1Fields(Map<String, String> aLogMap){
		aLogMap.put(UTLogFieldsScheme.SDKTYPE.toString(),"mini");
	}

	private static void _fillReservesFields(Map<String, String> aLogMap) {

		Map<String, String> lReservesMap = new HashMap<String, String>();

		// TRACK_ID
		if (aLogMap.containsKey(UTFields.TRACK_ID)) {
			String lValue = aLogMap.get(UTFields.TRACK_ID);
			aLogMap.remove(UTFields.TRACK_ID);
			if (!UTMCStringUtils.isEmpty(lValue)) {
				lReservesMap.put("_tkid", lValue);
			}
		}

		// 把reservesMap转换成String
		if (lReservesMap.size() > 0) {
			// 在改写reserves的值
			aLogMap.put(UTLogFieldsScheme.RESERVES.toString(),
					UTMCStringUtils.convertMapToString(lReservesMap));
		}

		if (!aLogMap.containsKey(UTLogFieldsScheme.PAGE.toString())) {
			aLogMap.put(UTLogFieldsScheme.PAGE.toString(), "UT");
		}

	}
	
	/**
	 * 页面开始展现
	 * @param aPageObject	页面对象，如Activity、Fragment等
	 */
	public void pageAppear(Object aPageObject){
		UTPageHitHelper.getInstance().pageAppear(aPageObject);
	}
	
	/**
	 * 页面展现（不被skip，直接生效）
	 * 定制时使用
	 * @param aPageObject
	 */
	public void pageAppearDonotSkip(Object aPageObject){
		UTPageHitHelper.getInstance().pageAppear(aPageObject,null,true);
	}
	
	/**
	 * 页面展现（不被skip，直接生效）
	 * 定制时使用
	 * @param aPageObject
	 */
	public void pageAppearDonotSkip(Object aPageObject,String aCustomPageName){
		UTPageHitHelper.getInstance().pageAppear(aPageObject,aCustomPageName,true);
	}
	
	/**
	 * 页面开始展现
	 * @param aPageObject		页面对象，如Activity、Fragment等
	 * @param aCustomPageName	自定义采集的页面名称，默认为类名（Activity后缀会被去除）
	 */
	public void pageAppear(Object aPageObject,String aCustomPageName){
		UTPageHitHelper.getInstance().pageAppear(aPageObject, aCustomPageName);
	}
	
	/**
	 * 页面开始消失
	 * @param aPageObject	页面对象，如Activity、Fragment等
	 */
	public void pageDisAppear(Object aPageObject){
		UTPageHitHelper.getInstance().pageDisAppear(aPageObject);
	}
	
	/**
	 * 更新参数到下一个页面时间上
	 * @param aProperties	需要更新的参数列表
	 */
	public void updateNextPageProperties(Map<String,String> aProperties){
		UTPageHitHelper.getInstance().updateNextPageProperties(aProperties);
	}
	
	/**
	 * 更新页面名称
	 * @param aPageObject	页面对象，如Activity、Fragment等
	 * @param aPageName	 	自定义的页面名称
	 */
	public void updatePageName(Object aPageObject,String aPageName){
		UTPageHitHelper.getInstance().updatePageName(aPageObject, aPageName);
	}
	
	/**
	 * 把一些属性对作为页面事件的参数
	 * @param aPageObject	页面对象，如Activity、Fragment等
	 * @param aProperties   属性列表
	 */
	public void updatePageProperties(Object aPageObject,Map<String, String> aProperties){
		UTPageHitHelper.getInstance().updatePageProperties(aPageObject, aProperties);
	}
	
	/**
	 * 更新页面状态
	 * @param aPageObject	页面对象，如Activity、Fragment等
	 * @param aPageStatus
	 */
	public void updatePageStatus(Object aPageObject,UTPageStatus aPageStatus){
		UTPageHitHelper.getInstance().updatePageStatus(aPageObject, aPageStatus);
	}
	
	/**
	 * 更新页面对应的地址uri
	 * @param aPageObject	页面对象，如Activity、Fragment等
	 * @param aUrl			页面对应的地址url
	 */
	public void updatePageUrl(Object aPageObject,Uri aUri){
		UTPageHitHelper.getInstance().updatePageUrl(aPageObject, aUri);
	}
	
	public void addTPKItem(UTTPKItem aTPKItem){
		if(null != aTPKItem){
			UTMCTPKBiz.getInstance().addTPKItem(aTPKItem);
		}
	}
	
	/**
	 * 设置这个页面不采集(仅能在onReasume方法中调用)
	 * @param aPageObject	页面对象
	 */
	public void skipPage(Object aPageObject){
		UTPageHitHelper.getInstance().skipPage(aPageObject);
	}
	
}
