package com.ut.mini.core.appstatus;

import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

import android.annotation.TargetApi;
import android.app.Activity;
import android.app.Application;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;

import com.ut.mini.module.UTMCScheduleMgr;

@TargetApi(Build.VERSION_CODES.ICE_CREAM_SANDWICH)
public class UTMCAppStatusMonitor implements Application.ActivityLifecycleCallbacks {

	private int mActivitiesActive = 0;
	private boolean mIsInForeground = false;
	private ScheduledFuture<?> mApplicationStatusScheduledFuture = null;
	private Object mApplicationStatusLockObj = new Object();
	private List<UTMCAppStatusCallbacks> mAppStatusCallbacksList = new LinkedList<UTMCAppStatusCallbacks>();
	private Object mAppStatusCallbacksLockObj = new Object();
	private static UTMCAppStatusMonitor s_instance = null;
	
	private UTMCAppStatusMonitor(){
		
	}
	
	public synchronized static UTMCAppStatusMonitor getInstance(){
		if(null == s_instance){
			s_instance = new UTMCAppStatusMonitor();
		}
		return s_instance;
	}

	public void registerAppStatusCallbacks(UTMCAppStatusCallbacks aCallbacks) {
		if (null != aCallbacks) {
			synchronized (mAppStatusCallbacksLockObj) {
				mAppStatusCallbacksList.add(aCallbacks);
			}
		}
	}

	public void unregisterAppStatusCallbacks(UTMCAppStatusCallbacks aCallbacks) {
		if (null != aCallbacks) {
			synchronized (mAppStatusCallbacksLockObj) {
				mAppStatusCallbacksList.remove(aCallbacks);
			}
		}
	}

	private void _clearApplicationStatusCheckExistingTimer() {
		synchronized (mApplicationStatusLockObj) {
			if (mApplicationStatusScheduledFuture != null) {
				mApplicationStatusScheduledFuture.cancel(true);
			}
		}
	}

	@Override
	public void onActivityCreated(Activity activity, Bundle savedInstanceState) {
		synchronized (mAppStatusCallbacksLockObj) {
			for (UTMCAppStatusCallbacks lCallback : mAppStatusCallbacksList) {
				lCallback.onActivityCreated(activity,savedInstanceState);
			}
		}
	}

	@Override
	public void onActivityDestroyed(Activity activity) {
		synchronized (mAppStatusCallbacksLockObj) {
			for (UTMCAppStatusCallbacks lCallback : mAppStatusCallbacksList) {
				lCallback.onActivityDestroyed(activity);
			}
		}
	}

	@Override
	public void onActivityPaused(Activity activity) {
		synchronized (mAppStatusCallbacksLockObj) {
			for (UTMCAppStatusCallbacks lCallback : mAppStatusCallbacksList) {
				lCallback.onActivityPaused(activity);
			}
		}
	}

	@Override
	public void onActivityResumed(Activity activity) {
		synchronized (mAppStatusCallbacksLockObj) {
			for (UTMCAppStatusCallbacks lCallback : mAppStatusCallbacksList) {
				lCallback.onActivityResumed(activity);
			}
		}
	}

	@Override
	public void onActivitySaveInstanceState(Activity activity, Bundle outState) {
		synchronized (mAppStatusCallbacksLockObj) {
			for (UTMCAppStatusCallbacks lCallback : mAppStatusCallbacksList) {
				lCallback.onActivitySaveInstanceState(activity, outState);
			}
		}
	}

	// 下面代码来源Google
	@Override
	public void onActivityStarted(Activity activity) {
		_clearApplicationStatusCheckExistingTimer();
		this.mActivitiesActive += 1;
		// 如果是后台 or 刚启动，那么我们设定定时器到30s
		if (!this.mIsInForeground) {
			// switch foreground
			synchronized (mAppStatusCallbacksLockObj) {
				for (UTMCAppStatusCallbacks lCallback : mAppStatusCallbacksList) {
					lCallback.onSwitchForeground();
				}
			}
		}
		this.mIsInForeground = true;
	}

	@Override
	public void onActivityStopped(Activity activity) {
		this.mActivitiesActive -= 1;
		if (this.mActivitiesActive == 0) {

			_clearApplicationStatusCheckExistingTimer();

			mApplicationStatusScheduledFuture = UTMCScheduleMgr.getInstance().schedule(
					new NotInForegroundRunnable(), 1000, TimeUnit.MILLISECONDS);
		}
	}

	private class NotInForegroundRunnable implements Runnable {

		private NotInForegroundRunnable() {

		}

		public void run() {
			// 因为没有调用，导致Path异步测试组件，爆出不通过。
			mIsInForeground = false;
			// switch background
			synchronized (mAppStatusCallbacksLockObj) {
				for (UTMCAppStatusCallbacks lCallback : mAppStatusCallbacksList) {
					lCallback.onSwitchBackground();
				}
			}
		}
	}

}
