
package com.ut.mini.utils;

import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.ProtocolException;
import java.net.URL;
import java.util.Map;
import java.util.Set;
import java.util.zip.GZIPInputStream;

/**
 * Http 请求
 * 
 * @author daichenjie Update:增加链路复用支持
 */
public final class UTMCHttpUtils {

    // Post 内容 头
    private final static String POST_Field_TOP = "--GJircTeP\r\nContent-Disposition: form-data; name=\"%s\"; filename=\"%s\"\r\nContent-Type: application/octet-stream \r\n\r\n";
    // Post 内容 底
    private final static String POST_Field_BOTTOM = "--GJircTeP--\r\n";
    /**
     * 请求方式：Get
     */
    public final static int HTTP_REQ_TYPE_GET = 1;
    /**
     * 请求方式：Post
     */
    public final static int HTTP_REQ_TYPE_POST_FORM_DATA = 2;

    public final static int HTTP_REQ_TYPE_POST_URL_ENCODED = 3;

    /**
     * 最大超时时间
     */
    public final static int MAX_CONNECTION_TIME_OUT = 10000;
    /**
     * 最大读取数据的
     */
    public final static int MAX_READ_CONNECTION_STREAM_TIME_OUT = 60000;

    static {
        System.setProperty("http.keepAlive", "true");
    }

    /**
     * Http请求
     * 
     * @param requestType 请求类型
     * @param aUrl 请求地址
     * @param aData 请求数据
     * @return Response结果
     */
    public static byte[] sendRequest(int requestType, String aUrl,
            Map<String, Object> pMap, boolean openGzip) {
        // 如果请求的URL为空或方法不为Get，Post
        if (UTMCStringUtils.isEmpty(aUrl)) {
            return null;
        }
        URL url = null;
        HttpURLConnection conn = null;
        try {
            url = new URL(aUrl);
            conn = (HttpURLConnection) url.openConnection();
        } catch (MalformedURLException e1) {
            e1.printStackTrace();
            return null;
        } catch (IOException e1) {
            e1.printStackTrace();
            return null;
        }

        if (conn != null) {
            if (requestType == HTTP_REQ_TYPE_POST_FORM_DATA
                    || requestType == HTTP_REQ_TYPE_POST_URL_ENCODED) {
                conn.setDoOutput(true);
            }
            conn.setDoInput(true);
            try {
                if (requestType == HTTP_REQ_TYPE_POST_FORM_DATA
                        || requestType == HTTP_REQ_TYPE_POST_URL_ENCODED) {
                    conn.setRequestMethod("POST");
                } else {
                    conn.setRequestMethod("GET");
                }
            } catch (ProtocolException e) {
                e.printStackTrace();
                return null;
            }
            conn.setUseCaches(false);
            conn.setConnectTimeout(MAX_CONNECTION_TIME_OUT);
            conn.setReadTimeout(MAX_READ_CONNECTION_STREAM_TIME_OUT);
            conn.setRequestProperty("Connection", "Keep-Alive");
            if (openGzip) {
                conn.setRequestProperty("Accept-Encoding", "gzip,deflate");
            }
            conn.setInstanceFollowRedirects(true);

            byte[] data = null;
            // 设置内容长度以及类型
            if (requestType == HTTP_REQ_TYPE_POST_FORM_DATA
                    || requestType == HTTP_REQ_TYPE_POST_URL_ENCODED) {

                if (requestType == HTTP_REQ_TYPE_POST_FORM_DATA) {
                    conn.setRequestProperty("Content-Type",
                            "multipart/form-data; boundary=GJircTeP");
                } else if (requestType == HTTP_REQ_TYPE_POST_URL_ENCODED) {
                    conn.setRequestProperty("Content-Type",
                            "application/x-www-form-urlencoded");
                }

                int contentLength = 0;
                if (null != pMap && pMap.size() > 0) {
                    ByteArrayOutputStream lBaos = new ByteArrayOutputStream();
                    Set<String> keys = pMap.keySet();
     
                    String[] lKeysArr = new String[keys.size()];  
                    keys.toArray(lKeysArr); 
                    
                    lKeysArr = UTMCKeyArraySorter.getInstance().sortResourcesList(lKeysArr, true);

                    for (String key : lKeysArr) {
                        if (requestType == HTTP_REQ_TYPE_POST_FORM_DATA) {
                            byte[] lValue = (byte[]) pMap.get(key);
                            if (null != lValue) {
                                try {
                                    lBaos.write(String.format(POST_Field_TOP,
                                            key, key).getBytes());
                                    lBaos.write(lValue);
                                    lBaos.write("\r\n".getBytes());
                                } catch (IOException e) {
                                    e.printStackTrace();
                                }
                            }
                        } else if (requestType == HTTP_REQ_TYPE_POST_URL_ENCODED) {
                            String lValue = (String) pMap.get(key);
                            if (lBaos.size() > 0) {
                                try {
                                    lBaos.write(("&" + key + "=" + lValue).getBytes());
                                } catch (IOException e) {
                                    e.printStackTrace();
                                }
                            } else {
                                try {
                                    lBaos.write((key + "=" + lValue).getBytes());
                                } catch (IOException e) {
                                    e.printStackTrace();
                                }
                            }
                        }
                    }
                    try {
                        if (requestType == HTTP_REQ_TYPE_POST_FORM_DATA){
                                lBaos.write(POST_Field_BOTTOM.getBytes());
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                    data = lBaos.toByteArray();
                }
                if (data != null) {
                    contentLength = data.length;
                }
                conn.setRequestProperty("Content-Length",
                        String.valueOf(contentLength));
            }

            // 建立链接
            DataOutputStream out = null;
            try {
                conn.connect();
                // 写请求流
                if ((requestType == HTTP_REQ_TYPE_POST_FORM_DATA || requestType == HTTP_REQ_TYPE_POST_URL_ENCODED)
                        && (data != null && data.length > 0)) {
                    out = new DataOutputStream(conn.getOutputStream());
                    out.write(data);
                    out.flush();
                }
            } catch (Exception e) { // 修正网络权限导致的崩溃
                e.printStackTrace();
                return null;
            } finally {
                if (out != null) {
                    try {
                        out.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                    out = null;
                }
            }

            // 读取数据流
            InputStream dis = null;
            // 分块读取数据流
            ByteArrayOutputStream bs = new ByteArrayOutputStream();
            try {
                if (openGzip && "gzip".equals(conn.getContentEncoding())) {
                    dis = new GZIPInputStream(conn.getInputStream());
                } else {
                    dis = new DataInputStream(conn.getInputStream());
                }
                long startTime = System.currentTimeMillis();
                byte[] buffer = new byte[2048];
                int i;
                while ((i = dis.read(buffer, 0, 2048)) != -1) {
                    bs.write(buffer, 0, i);
                    // 超时 终止
                    if (System.currentTimeMillis() - startTime > 10000) {
                        bs.reset();
                        break;
                    }
                }
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            } finally {
                if (dis != null) {
                    try {
                        dis.close();
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }

            if (bs.size() > 0) {
                return bs.toByteArray();
            } else {
                return null;
            }
        } else {
            return null;
        }
    }
}
